<?if (!defined("B_PROLOG_INCLUDED") || B_PROLOG_INCLUDED !== true) die();?><?

$onpay = new Onpay(CSalePaySystemAction::GetParamValue("Login"), CSalePaySystemAction::GetParamValue("SecretKey"));

if($onpay->check()) {
	die;
} else if($onpay->pay()) {
	$request = $onpay->getData();
	$pay_for = explode('_', $request['pay_for']);
	if (!($arOrder = CSaleOrder::GetByID($pay_for))) {
		die;
	}

	$amount = $onpay->toFloat($request['payment']['amount']);

	$arFields = array(
		"PS_STATUS" => "Y",
		"PS_STATUS_CODE" => "-",
		"PS_STATUS_DESCRIPTION" => '',
		"PS_STATUS_MESSAGE" => '',
		"PS_SUM" => $amount,
		"PS_CURRENCY" => $arOrder["CURRENCY"],
		"PS_RESPONSE_DATE" => Date(CDatabase::DateFormatToPHP(CLang::GetDateFormat("FULL", LANG))),
		"USER_ID" => $arOrder["USER_ID"]
	);

	if ($onpay->toFloat($arOrder["PRICE"]) == $amount && $arOrder['PAYED'] != 'Y') {
		CSaleOrder::PayOrder($arOrder["ID"], "Y");
		CSaleOrder::Update($arOrder["ID"], $arFields);
	}
}

class Onpay
{
  public $login;

  public $secret_key;

  public function __construct($login, $secret_key)
  {
    $this->login = $login;
    $this->secret_key = $secret_key;
  }

  public function check() {
    $request = $this->getData();

    if(!isset($request['pay_for']) || $request['type'] != 'check') {
      return false;
    }

    $check = array(
      'type' => 'check',
      'pay_for' => $request['pay_for'],
      'amount' => $this->toFloat($request['amount']),
      'currency' => trim($request['way']),
      'mode' => trim($request['mode']),
      'key' => $this->secret_key,
    );
    $check['signature_string'] = implode(";", $check);
    $check['signature'] = sha1($check['signature_string']);
    $checkOut = array(
      'type' => 'check',
      'status' => 'false',
      'pay_for' => $request['pay_for'],
      'key' => $this->secret_key,
    );

    if($this->validate($request, $check['signature'])) {
      $checkOut['status'] = 'true';
    }
    $this->response($checkOut);

    return $checkOut['status'] == 'true' ? true : false;
  }

  public function pay() {
    $request = $this->getData();

    if(!isset($request['pay_for']) || $request['type'] != 'pay') {
      return false;
    }

    $pay = array(
      'type' => 'pay',
      'pay_for' => $request['pay_for'],
      'payment.amount' => $this->toFloat($request['payment']['amount']),
      'payment.currency' => trim($request['payment']['way']),
      'amount' => $this->toFloat($request['balance']['amount']),
      'currency' => trim($request['balance']['way']),
      'key' => $this->secret_key,
    );
    $pay['signature_string'] = implode(";", $pay);
    $pay['signature'] = sha1($pay['signature_string']);
    $payOut = array(
      'type' => 'pay',
      'status' => 'false',
      'pay_for' => $request['pay_for'],
      'key' => $this->secret_key,
    );

    if($this->validate($request, $pay['signature'])) {
      $payOut['status'] = 'true';
    }
    $this->response($payOut);

    return $payOut['status'] == 'true' ? true : false;
  }

  public function response($response) {
    $response['signature_string'] = implode(";", $response);
    $response['signature'] = sha1($response['signature_string']);
    $out = "{\"status\":{$response['status']},\"pay_for\":\"{$response['pay_for']}\",\"signature\":\"{$response['signature']}\"}";

    header("Content-type: application/json; charset=utf-8");
    echo iconv("cp1251", "utf-8", $out);
  }

  public function validate($request, $signature) {
    $request['pay_for'] = intval($request['pay_for']);
    if($request['signature'] != $signature) {
      return false;
    }
    return true;
  }

  public function getData() {
    $ret = false;
    if(function_exists('json_decode')) {
      if(isset($GLOBALS['__inputData'])) {
        $ret = $GLOBALS['__inputData'];
      } elseif($hSource = fopen('php://input', 'r')) {
        $input = "";
        while (!feof($hSource)) {
          $input .= fread($hSource, 1024);
        }
        fclose($hSource);
        $input = trim($input);

        $ret = json_decode($input, true);
        if(is_null($ret)) $ret = json_decode(iconv("cp1251", "utf-8", $input), true);

        $GLOBALS['__inputData'] = $ret;
      }
    }

    return $ret;
  }

  public function toFloat($sum)
  {
    $sum = floatval($sum);
    if (strpos($sum, ".")) {
      $sum = round($sum, 2);
    } else {
      $sum = $sum . ".0";
    }
    return $sum;
  }
}
?>
